package net.blumia.pcmdroid.ui.screen.main

import android.net.Uri
import androidx.compose.foundation.*
import androidx.compose.foundation.gestures.scrollable
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.*
import androidx.compose.material.icons.rounded.FeaturedPlayList
import androidx.compose.material.icons.rounded.PlayArrow
import androidx.compose.material.icons.rounded.PlayCircle
import androidx.compose.material.icons.rounded.PlayCircleOutline
import androidx.compose.runtime.*
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.layout.VerticalAlignmentLine
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import kotlinx.coroutines.launch
import net.blumia.pcmdroid.R
import net.blumia.pcmdroid.model.Folder
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.model.Song

@Preview(showBackground = true)
@OptIn(ExperimentalMaterialApi::class)
@Composable
fun NowPlaying() {
    Column(
        modifier = Modifier
            .fillMaxWidth(),
    ) {
        LinearProgressIndicator(
            modifier = Modifier.fillMaxWidth(),
            progress = 0f,
        )

        Row(
            modifier = Modifier
                .fillMaxWidth()
                .height(56.dp),
            verticalAlignment = Alignment.CenterVertically,
        ) {
            Image(
                painter = painterResource(id = R.drawable.ic_launcher_background),
                contentDescription = "Album cover",
            )

            ListItem(
                text = {
                    Text(
                        text = "Not playing at all.",
                    )
                },
                trailing = {
                    IconButton(onClick = { /*TODO*/ }) {
                        Icon(imageVector = Icons.Rounded.PlayCircleOutline, contentDescription = "Menu")
                    }
                }
            )
        }
    }
}

@Preview(showBackground = true)
@OptIn(ExperimentalMaterialApi::class)
@Composable
fun DrawerContent(
    servers: List<Server> = listOf(),
    drawerFolders: List<Folder> = listOf(),
    addServerActionTriggered: () -> Unit = {},
    onServerItemIconClicked: (Server) -> Unit = {},
    onDrawerFolderItemClicked: (Folder) -> Unit = {},
    settingsActionTriggered: () -> Unit = {},
) {
    Row(modifier = Modifier.fillMaxSize()) {
        var selectedItem by remember { mutableStateOf<Int?>(0) }
        NavigationRail {
            servers.forEachIndexed { index, server ->
                NavigationRailItem(
                    icon = {
                        Icon(
                            Icons.Filled.LibraryMusic, server.name
                        )
                    },
                    label = {
                        Text(server.name)
                    },
                    selected = selectedItem == index,
                    onClick = {
                        onServerItemIconClicked(server)
                    }
                )
            }

            NavigationRailItem(
                icon = { Icon(Icons.Filled.AddCircle, null) },
                label = { Text(stringResource(id = R.string.add)) },
                selected = false,
                onClick = addServerActionTriggered,
            )

            Spacer(modifier = Modifier.weight(1f))

            NavigationRailItem(
                icon = { Icon(Icons.Filled.Settings, null) },
                label = { Text(stringResource(id = R.string.settings)) },
                selected = false,
                onClick = settingsActionTriggered,
            )
        }

        Column(modifier = Modifier.weight(1f)) {
            ListItem(
                text = {
                    Text(
                        text = "No server selected",
                        style = MaterialTheme.typography.subtitle1
                    )
                },
                trailing = {
                    IconButton(onClick = { /*TODO*/ }) {
                        Icon(imageVector = Icons.Filled.MoreVert, contentDescription = "Menu")
                    }
                }
            )

            Divider()

            drawerFolders.forEach { folder ->
                ListItem(
                    modifier = Modifier.clickable{
                        onDrawerFolderItemClicked(folder)
                    },
                    text = {
                        Text(
                            text = folder.displayName(),
                            style = MaterialTheme.typography.subtitle1
                        )
                    }
                )
            }
        }
    }
}

@Preview(showBackground = true)
@OptIn(ExperimentalMaterialApi::class)
@Composable
fun MainPlayer(
    servers: List<Server> = listOf(),
    drawerFolders: List<Folder> = listOf(),
    folders: List<Folder> = listOf(),
    songs: List<Song> = listOf(),
    addServerActionTriggered: () -> Unit = {},
    onServerItemIconClicked: (Server) -> Unit = {},
    onDrawerFolderItemClicked: (Folder) -> Unit = {},
    settingsActionTriggered: () -> Unit = {},
) {
    val drawerState = rememberDrawerState(initialValue = DrawerValue.Closed)
    val drawerScope = rememberCoroutineScope()
    ModalDrawer(
        drawerState = drawerState,
        drawerContent = {
            DrawerContent(
                servers = servers,
                drawerFolders = drawerFolders,
                addServerActionTriggered = addServerActionTriggered,
                onServerItemIconClicked = onServerItemIconClicked,
                onDrawerFolderItemClicked = onDrawerFolderItemClicked,
                settingsActionTriggered = settingsActionTriggered,
            )
        }
    ) {
        Scaffold(
            topBar = {
                TopAppBar(
                    title = { Text(stringResource(id = R.string.app_name)) },
                    navigationIcon = {
                        IconButton(onClick = {
                            drawerScope.launch {
                                drawerState.open()
                            }
                        }) {
                            Icon(imageVector = Icons.Filled.Menu, contentDescription = "Open drawer")
                        }
                    },
                    actions = {
                        IconButton(onClick = { /*TODO*/ }) {
                            Icon(imageVector = Icons.Filled.MoreVert, contentDescription = "Menu")
                        }
                    }
                )
            }
        ) {
            Column(
                modifier = Modifier.fillMaxSize()
            ) {
                Column(
                    modifier = Modifier
                        .weight(1f, fill = true)
                        .verticalScroll(rememberScrollState())
                ) {

                    val iconLeftPadding = 16.dp
                    val iconVerticalPadding = 12.dp
                    Row(
                        modifier = Modifier
                            .fillMaxWidth()
                            .horizontalScroll(rememberScrollState())
                            .padding(
                                horizontal = iconLeftPadding,
                                vertical = iconVerticalPadding,
                            ),
                        verticalAlignment = Alignment.CenterVertically,
                    ) {
                        val pathSeg = listOf("Deep", "Dark", "Folder")
                        pathSeg.forEach { seg ->
                            val isLastSeg = seg == pathSeg.last()

                            Text(
                                text = seg,
                                style = MaterialTheme.typography.body1,
                                color = if (isLastSeg) MaterialTheme.colors.primary else MaterialTheme.colors.onSurface
                            )

                            if (!isLastSeg) {
                                Icon(
                                    modifier = Modifier.padding(horizontal = 5.dp),
                                    imageVector = Icons.Filled.NavigateNext,
                                    contentDescription = null,
                                )
                            }
                        }
                    }

                    folders.forEach { folder ->
                        ListItem(
                            modifier = Modifier.clickable {  },
                            icon = {
                                Icon(
                                    imageVector = Icons.Filled.FolderOpen,
                                    contentDescription = null,
                                )
                            },
                            text = {
                                Text(folder.displayName())
                            }
                        )
                    }

                    songs.forEach { song ->
                        ListItem(
                            modifier = Modifier.clickable {  },
                            icon = {
                                Icon(
                                    imageVector = Icons.Filled.MusicNote,
                                    contentDescription = null
                                )
                            },
                            text = {
                                Text(song.displayName())
                            }
                        )
                    }
                }
                NowPlaying()
            }
        }
    }

}
